/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_mkc18.c                                                                 *
 *                                                                                              *
 *  Compilers:      mikroC                                                                      *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    mikroC for PIC18 specific functions overloads                               *
 *                                                                                              *
 *  History:        21.01.2009                                                                  *
 *                  21.03.2009 -    Bug fixed: OS_RI added into _OS_Dtimer_Delete               *
 *                  01.04.2009 -    All processor specific definitions moved from osa.c into    *
 *                                  this file                                                   *
 *                                                                                              *
 ************************************************************************************************
 */



#define OS_Init_ProcSpec()      // Empty macro for mikroC


volatile unsigned int _fsr      absolute 0xFE9;
volatile char         _indf     absolute 0xFEF;
volatile char         _postinc  absolute 0xFEE;
volatile char         _postdec  absolute 0xFED;
volatile char         _preinc   absolute 0xFEC;

volatile unsigned char _fsr1l   absolute 0xFE1;

volatile char         _pcl      absolute 0xFF9;
volatile char         _pclath   absolute 0xFFA;
volatile char         _pclatu   absolute 0xFFB;
volatile char         _status   absolute 0xFD8;
volatile char         _tosl     absolute 0xFFD;
volatile char         _tosh     absolute 0xFFE;
volatile char         _tosu     absolute 0xFFF;
volatile char         _bsr      absolute 0xFE0;
volatile char         _wreg     absolute 0xFE8;
volatile char         _intcon   absolute 0xFF2;
volatile char         _rcon     absolute 0xFD0;
volatile char         _stkptr   absolute 0xFFC;










void __OS_JumpToTask (void)
{
    _OS_SET_FSR_CUR_TASK();
    *((NEAR char*)&OS_state) = _postinc;
    asm {
        push
        movf    __postinc, w, 0
        movwf   __tosl
        movf    __postinc, w, 0
        movwf   __tosh
        movf    __postinc, w, 0
        movwf   __tosu
        return
    }
}


/************************************************************************************************
 *                                                                                              *
 *                           K E R N E L   F U N C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************/


void _OS_ClrReadySetClrCanContinue (void);  /* see description below    */

/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler with saving current task return point    *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPoint (void)
{
    _OS_SET_FSR_CUR_TASK();
    _OS_SavePC();
    _OS_bTaskReady = 1;
    _OS_POP18();
}



/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPointNoSave (void)                                          *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to scheduler without saving context                  *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPointNoSave (void)
{
    _OS_POP18();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReady (void)                                                  *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag and return to the sheduler with seaving   *
 *                  task return point                                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReady (void)
{
    _OS_temp = 0;
    _OS_ClrReadySetClrCanContinue();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReadySetCanContinue (void)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag, set bCanContinue flag and return to the  *
 *                  sheduler with seaving task return point                     *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReadySetCanContinue (void)
{
    _OS_temp = 0xFF;
    _OS_ClrReadySetClrCanContinue();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClrReadySetClrCanContinue (void)                                   *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Addition function. Called only by GOTO from:                *
 *                      OS_ClearReady()                                         *
 *                      OS_ClearReadySetCanContinue()                           *
 *                  Stack has pointer of task who called theese services.       *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void _OS_ClrReadySetClrCanContinue (void)
{
    _OS_POP18();

    OS_Flags.bEventError = 0;

    _OS_SET_FSR_CUR_TASK();
    _OS_SavePC();
    _OS_bTaskReady = 0;

    #if defined(OS_ENABLE_TTIMERS)
        _OS_bTaskCanContinue = 0;
        if (_OS_temp & 0x01) _OS_bTaskCanContinue = 1;
    #endif

}




















/************************************************************************************************
 *                                                                                              *
 *  MAKE FSR POINTER TO OS_CUR_TASK                                                             *
 *                                                                                              *
 *  (for code reducing)                                                                         *
 *                                                                                              *
 ************************************************************************************************/

void _OS_SET_FSR_CUR_TASK (void)
{
    _fsr = (_FSR_TYPE)_OS_CurTask;
}


















/************************************************************************************************
 *                                                                                              *
 *                            S Y S T E M   F U N C T I O N S                                   *
 *                                                                                              *
 ************************************************************************************************/







/*
 ********************************************************************************
 *                                                                              *
 *  char OS_DI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Disable interrupt with GIE saving                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      char - bit 7 = previous GIEH value,                         *
 *                         bit 6 = previous GIEL value                          *
 *                         bits 0..5 = 0                                        *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_DI_DEFINED
//-------------------------------------------------------------------------------

char OS_DI (void)
{
    char temp;
    temp = _intcon & 0xC0;
    _intcon &= 0x7F;
    if (RCON & 0x80) _intcon &= 0xBF;
    return temp;
}
/*
 ********************************************************************************
 *                                                                              *
 *  char OS_RI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Restore saved by OS_DI() GIEx states                        *
 *                                                                              *
 *  parameters:     char - bit 7 = previos GIE value, bits 0..6 = 0             *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_RI_DEFINED
//-------------------------------------------------------------------------------

void OS_RI (char temp)
{
    if (temp & 0x80) _intcon |= 0x80;
    if (RCON & 0x80) {
        if (temp & 0x40) _intcon |= 0x40;
    }
}





/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_CheckEvent (char bEvent)                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_CheckEvent_DEFINED
//------------------------------------------------------------------------------

void _OS_CheckEvent (_OST_INT_TYPE bEvent)
{

    OS_Flags.bTimeout = 0;
    _OS_SET_FSR_CUR_TASK();

    if (bEvent)
    {
        if (_OS_bTaskReady) {

            #ifdef OS_ENABLE_TTIMERS

                _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state
                _indf &= ~0x10;     // Clear bDelay in task descriptor

            #endif

            return;
        }

        _OS_bTaskReady = 1;

    } else {
        _OS_bTaskReady = 0;
    }

    #ifdef OS_ENABLE_TTIMERS

        if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
        {
            OS_Flags.bTimeout = 1;
            return;
        }

    #endif

    _OS_POP18();
}








